@@ -1,35 +1,37 @@
 #include "format.h"
 #include "error.h"
 #include "util.h"
-#ifdef USE_BOOST_PYTHON
-#include "py_eval.h"
-#endif
 
 #include <cstdlib>
-#include <ctime>
 
 namespace ledger {
 
-std::string truncated(const std::string& str, unsigned int width,
-		      const int style)
+format_t::elision_style_t format_t::elision_style = ABBREVIATE;
+int format_t::abbrev_length = 2;
+
+bool format_t::ansi_codes  = false;
+bool format_t::ansi_invert = false;
+
+std::string format_t::truncate(const std::string& str, unsigned int width,
+			       const bool is_account)
 {
   const int len = str.length();
   if (len <= width)
     return str;
 
-  assert(width < 254);
+  assert(width < 4095);
 
-  char buf[256];
+  char buf[4096];
 
-  switch (style) {
-  case 0:
+  switch (elision_style) {
+  case TRUNCATE_LEADING:
     // This method truncates at the beginning.
     std::strncpy(buf, str.c_str() + (len - width), width);
     buf[0] = '.';
     buf[1] = '.';
     break;
 
-  case 1:
+  case TRUNCATE_MIDDLE:
     // This method truncates in the middle.
     std::strncpy(buf, str.c_str(), width / 2);
     std::strncpy(buf + width / 2,
@@ -39,7 +41,52 @@ std::string truncated(const std::string& str, unsigned int width,
     buf[width / 2] = '.';
     break;
 
-  case 2:
+  case ABBREVIATE:
+    if (is_account) {
+      std::list<std::string> parts;
+      std::string::size_type beg = 0;
+      for (std::string::size_type pos = str.find(':');
+	   pos != std::string::npos;
+	   beg = pos + 1, pos = str.find(':', beg))
+	parts.push_back(std::string(str, beg, pos - beg));
+      parts.push_back(std::string(str, beg));
+
+      std::string result;
+      int newlen = len;
+      for (std::list<std::string>::iterator i = parts.begin();
+	   i != parts.end();
+	   i++) {
+	// Don't contract the last element
+	std::list<std::string>::iterator x = i;
+	if (++x == parts.end()) {
+	  result += *i;
+	  break;
+	}
+
+	if (newlen > width) {
+	  result += std::string(*i, 0, abbrev_length);
+	  result += ":";
+	  newlen -= (*i).length() - abbrev_length;
+	} else {
+	  result += *i;
+	  result += ":";
+	}
+      }
+
+      if (newlen > width) {
+	// Even abbreviated its too big to show the last account, so
+	// abbreviate all but the last and truncate at the beginning.
+	std::strncpy(buf, result.c_str() + (result.length() - width), width);
+	buf[0] = '.';
+	buf[1] = '.';
+      } else {
+	std::strcpy(buf, result.c_str());
+      }
+      break;
+    }
+    // fall through...
+
+  case TRUNCATE_TRAILING:
     // This method truncates at the end (the default).
     std::strncpy(buf, str.c_str(), width - 2);
     buf[width - 2] = '.';
@@ -71,8 +118,6 @@ std::string partial_account_name(const account_t& account)
   return name;
 }
 
-std::string format_t::date_format = "%Y/%m/%d";
-
 element_t * format_t::parse_elements(const std::string& fmt)
 {
   std::auto_ptr<element_t> result;
@@ -120,8 +165,15 @@ element_t * format_t::parse_elements(const std::string& fmt)
     }
 
     ++p;
-    if (*p == '-') {
-      current->align_left = true;
+    while (*p == '!' || *p == '-') {
+      switch (*p) {
+      case '-':
+	current->flags |= ELEMENT_ALIGN_LEFT;
+	break;
+      case '!':
+	current->flags |= ELEMENT_HIGHLIGHT;
+	break;
+      }
       ++p;
     }
 
@@ -162,28 +214,12 @@ element_t * format_t::parse_elements(const std::string& fmt)
 	p++;
       }
       if (*p != ')')
-	throw format_error("Missing ')'");
+	throw new format_error("Missing ')'");
 
       current->type = element_t::VALUE_EXPR;
-      try {
-	current->val_expr = parse_value_expr(std::string(b, p));
-      }
-      catch (value_expr_error& err) {
-	throw value_expr_error(std::string("In format expression '") +
-			       std::string(b, p) + "': " + err.what());
-      }
-      break;
-    }
 
-    case '@': {
-      const char * s = ++p;
-      while (*p && *p != '(')
-	p++;
-      if (*p && *++p != ')')
-	throw format_error("Missing ')'");
-
-      current->type  = element_t::INTERP_FUNC;
-      current->chars = std::string(s, (p - s) - 1);
+      assert(! current->val_expr);
+      current->val_expr = std::string(b, p);
       break;
     }
 
@@ -199,24 +235,43 @@ element_t * format_t::parse_elements(const std::string& fmt)
 	p++;
       }
       if (*p != ']')
-	throw format_error("Missing ']'");
+	throw new format_error("Missing ']'");
 
       current->type  = element_t::DATE_STRING;
       current->chars = std::string(b, p);
       break;
     }
 
+    case 'x':
+      switch (*++p) {
+      case 'B': current->type = element_t::XACT_BEG_POS; break;
+      case 'b': current->type = element_t::XACT_BEG_LINE; break;
+      case 'E': current->type = element_t::XACT_END_POS; break;
+      case 'e': current->type = element_t::XACT_END_LINE; break;
+      case '\0':
+	goto END;
+      }
+      break;
+
+    case 'd':
+      current->type  = element_t::COMPLETE_DATE_STRING;
+      current->chars = datetime_t::output_format;
+      break;
     case 'D':
       current->type  = element_t::DATE_STRING;
-      current->chars = format_t::date_format;
+      current->chars = datetime_t::output_format;
       break;
 
     case 'S': current->type = element_t::SOURCE; break;
-    case 'B': current->type = element_t::BEG_POS; break;
-    case 'E': current->type = element_t::END_POS; break;
+    case 'B': current->type = element_t::ENTRY_BEG_POS; break;
+    case 'b': current->type = element_t::ENTRY_BEG_LINE; break;
+    case 'E': current->type = element_t::ENTRY_END_POS; break;
+    case 'e': current->type = element_t::ENTRY_END_LINE; break;
     case 'X': current->type = element_t::CLEARED; break;
+    case 'Y': current->type = element_t::ENTRY_CLEARED; break;
     case 'C': current->type = element_t::CODE; break;
     case 'P': current->type = element_t::PAYEE; break;
+    case 'W': current->type = element_t::OPT_ACCOUNT; break;
     case 'a': current->type = element_t::ACCOUNT_NAME; break;
     case 'A': current->type = element_t::ACCOUNT_FULLNAME; break;
     case 't': current->type = element_t::AMOUNT; break;
@@ -229,6 +284,7 @@ element_t * format_t::parse_elements(const std::string& fmt)
     }
   }
 
+ END:
   if (q != buf) {
     if (! result.get()) {
       result.reset(new element_t);
@@ -244,13 +300,34 @@ element_t * format_t::parse_elements(const std::string& fmt)
   return result.release();
 }
 
+namespace {
+  inline void mark_red(std::ostream& out, const element_t * elem) {
+    out.setf(std::ios::left);
+    out.width(0);
+    out << "\e[31m";
+
+    if (elem->flags & ELEMENT_ALIGN_LEFT)
+      out << std::left;
+    else
+      out << std::right;
+
+    if (elem->min_width > 0)
+      out.width(elem->min_width);
+  }
+
+  inline void mark_plain(std::ostream& out) {
+    out << "\e[0m";
+  }
+}
+
 void format_t::format(std::ostream& out_str, const details_t& details) const
 {
   for (const element_t * elem = elements; elem; elem = elem->next) {
     std::ostringstream out;
+    std::string name;
     bool ignore_max_width = false;
 
-    if (elem->align_left)
+    if (elem->flags & ELEMENT_ALIGN_LEFT)
       out << std::left;
     else
       out << std::right;
@@ -266,33 +343,82 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
     case element_t::AMOUNT:
     case element_t::TOTAL:
     case element_t::VALUE_EXPR: {
-      value_expr_t * expr = NULL;
+      value_expr calc;
       switch (elem->type) {
-      case element_t::AMOUNT:     expr = amount_expr.get(); break;
-      case element_t::TOTAL:      expr = total_expr.get(); break;
-      case element_t::VALUE_EXPR: expr = elem->val_expr; break;
+      case element_t::AMOUNT:     calc = amount_expr; break;
+      case element_t::TOTAL:      calc = total_expr; break;
+      case element_t::VALUE_EXPR: calc = elem->val_expr; break;
       default:
 	assert(0);
 	break;
       }
-      if (! expr)
+      if (! calc)
 	break;
 
       value_t     value;
       balance_t * bal = NULL;
 
-      expr->compute(value, details);
+      calc->compute(value, details);
+
+      if (! amount_t::keep_price ||
+	  ! amount_t::keep_date ||
+	  ! amount_t::keep_tag) {
+	switch (value.type) {
+	case value_t::AMOUNT:
+	case value_t::BALANCE:
+	case value_t::BALANCE_PAIR:
+	  value = value.strip_annotations();
+	  break;
+	default:
+	  break;
+	}
+      }
+
+      bool highlighted = false;
 
       switch (value.type) {
       case value_t::BOOLEAN:
-	out << (*((bool *) value.data) ? "1" : "0");
+	out << (*((bool *) value.data) ? "true" : "false");
 	break;
+
       case value_t::INTEGER:
-	out << *((unsigned int *) value.data);
+	if (ansi_codes && elem->flags & ELEMENT_HIGHLIGHT) {
+	  if (ansi_invert) {
+	    if (*((long *) value.data) > 0) {
+	      mark_red(out, elem);
+	      highlighted = true;
+	    }
+	  } else {
+	    if (*((long *) value.data) < 0) {
+	      mark_red(out, elem);
+	      highlighted = true;
+	    }
+	  }
+	}
+	out << *((long *) value.data);
 	break;
+
+      case value_t::DATETIME:
+	out << *((datetime_t *) value.data);
+	break;
+
       case value_t::AMOUNT:
+	if (ansi_codes && elem->flags & ELEMENT_HIGHLIGHT) {
+	  if (ansi_invert) {
+	    if (*((amount_t *) value.data) > 0) {
+	      mark_red(out, elem);
+	      highlighted = true;
+	    }
+	  } else {
+	    if (*((amount_t *) value.data) < 0) {
+	      mark_red(out, elem);
+	      highlighted = true;
+	    }
+	  }
+	}
 	out << *((amount_t *) value.data);
 	break;
+
       case value_t::BALANCE:
 	bal = (balance_t *) value.data;
 	// fall through...
@@ -301,37 +427,54 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
 	if (! bal)
 	  bal = &((balance_pair_t *) value.data)->quantity;
 
+	if (ansi_codes && elem->flags & ELEMENT_HIGHLIGHT) {
+	  if (ansi_invert) {
+	    if (*bal > 0) {
+	      mark_red(out, elem);
+	      highlighted = true;
+	    }
+	  } else {
+	    if (*bal < 0) {
+	      mark_red(out, elem);
+	      highlighted = true;
+	    }
+	  }
+	}
 	bal->write(out, elem->min_width,
-		   (elem->max_width > 0 ? elem->max_width : elem->min_width));
+		   (elem->max_width > 0 ?
+		    elem->max_width : elem->min_width));
+
 	ignore_max_width = true;
 	break;
       default:
 	assert(0);
 	break;
       }
+
+      if (highlighted)
+	mark_plain(out);
       break;
     }
 
     case element_t::OPT_AMOUNT:
       if (details.xact) {
 	std::string disp;
-	bool        use_disp = false;
+	bool use_disp = false;
 
 	if (details.xact->cost && details.xact->amount) {
-	  amount_t unit_cost = *details.xact->cost / details.xact->amount;
-
-	  commodity_t& comm(unit_cost.commodity());
-	  bool has_flag = comm.flags & COMMODITY_STYLE_VARIABLE;
-	  if (! has_flag)
-	    unit_cost.commodity().flags |= COMMODITY_STYLE_VARIABLE;
-
 	  std::ostringstream stream;
-	  stream << details.xact->amount << " @ " << unit_cost;
+	  if (! details.xact->amount_expr.expr.empty())
+	    stream << details.xact->amount_expr.expr;
+	  else
+	    stream << details.xact->amount.strip_annotations();
+
+	  if (! details.xact->cost_expr.empty())
+	    stream << details.xact->cost_expr;
+	  else
+	    stream << " @ " << amount_t(*details.xact->cost /
+					details.xact->amount).unround();
 	  disp = stream.str();
 	  use_disp = true;
-
-	  if (! has_flag)
-	    unit_cost.commodity().flags &= ~COMMODITY_STYLE_VARIABLE;
 	}
 	else if (details.entry) {
 	  unsigned int    xacts_count = 0;
@@ -354,10 +497,14 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
 		      first->amount == - last->amount);
 	}
 
-	if (! use_disp)
-	  out << details.xact->amount;
-	else
+	if (! use_disp) {
+	  if (! details.xact->amount_expr.expr.empty())
+	    out << details.xact->amount_expr.expr;
+	  else
+	    out << details.xact->amount.strip_annotations();
+	} else {
 	  out << disp;
+	}
       }
       break;
 
@@ -374,32 +521,117 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
       }
       break;
 
-    case element_t::BEG_POS:
+    case element_t::ENTRY_BEG_POS:
       if (details.entry)
 	out << (unsigned long)details.entry->beg_pos;
       break;
 
-    case element_t::END_POS:
+    case element_t::ENTRY_BEG_LINE:
+      if (details.entry)
+	out << details.entry->beg_line;
+      break;
+
+    case element_t::ENTRY_END_POS:
       if (details.entry)
 	out << (unsigned long)details.entry->end_pos;
       break;
 
-    case element_t::DATE_STRING:
-      if (details.entry && details.entry->date) {
-	char buf[256];
-	std::strftime(buf, 255, elem->chars.c_str(),
-		      std::localtime(&details.entry->date));
-	out << (elem->max_width == 0 ? buf : truncated(buf, elem->max_width));
+    case element_t::ENTRY_END_LINE:
+      if (details.entry)
+	out << details.entry->end_line;
+      break;
+
+    case element_t::XACT_BEG_POS:
+      if (details.xact)
+	out << (unsigned long)details.xact->beg_pos;
+      break;
+
+    case element_t::XACT_BEG_LINE:
+      if (details.xact)
+	out << details.xact->beg_line;
+      break;
+
+    case element_t::XACT_END_POS:
+      if (details.xact)
+	out << (unsigned long)details.xact->end_pos;
+      break;
+
+    case element_t::XACT_END_LINE:
+      if (details.xact)
+	out << details.xact->end_line;
+      break;
+
+    case element_t::DATE_STRING: {
+      datetime_t date;
+      if (details.xact)
+	date = details.xact->date();
+      else if (details.entry)
+	date = details.entry->date();
+
+      char buf[256];
+      std::strftime(buf, 255, elem->chars.c_str(), date.localtime());
+      out << (elem->max_width == 0 ? buf : truncate(buf, elem->max_width));
+      break;
+    }
+
+    case element_t::COMPLETE_DATE_STRING: {
+      datetime_t actual_date;
+      datetime_t effective_date;
+      if (details.xact) {
+	actual_date    = details.xact->actual_date();
+	effective_date = details.xact->effective_date();
+      }
+      else if (details.entry) {
+	actual_date    = details.entry->actual_date();
+	effective_date = details.entry->effective_date();
+      }
+
+      char abuf[256];
+      std::strftime(abuf, 255, elem->chars.c_str(), actual_date.localtime());
+
+      if (effective_date && effective_date != actual_date) {
+	char buf[512];
+	char ebuf[256];
+	std::strftime(ebuf, 255, elem->chars.c_str(),
+		      effective_date.localtime());
+
+	std::strcpy(buf, abuf);
+	std::strcat(buf, "=");
+	std::strcat(buf, ebuf);
+
+	out << (elem->max_width == 0 ? buf : truncate(buf, elem->max_width));
       } else {
-	out << " ";
+	out << (elem->max_width == 0 ? abuf : truncate(abuf, elem->max_width));
+      }
+      break;
+    }
+
+    case element_t::CLEARED:
+      if (details.xact) {
+	switch (details.xact->state) {
+	case transaction_t::CLEARED:
+	  out << "* ";
+	  break;
+	case transaction_t::PENDING:
+	  out << "! ";
+	  break;
+	}
       }
       break;
 
-    case element_t::CLEARED:
-      if (details.entry && details.entry->state == entry_t::CLEARED)
-	out << "* ";
-      else
-	out << "";
+    case element_t::ENTRY_CLEARED:
+      if (details.entry) {
+	transaction_t::state_t state;
+	if (details.entry->get_state(&state))
+	  switch (state) {
+	  case transaction_t::CLEARED:
+	    out << "* ";
+	    break;
+	  case transaction_t::PENDING:
+	    out << "! ";
+	    break;
+	  }
+      }
       break;
 
     case element_t::CODE: {
@@ -416,8 +648,8 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
     case element_t::PAYEE:
       if (details.entry)
 	out << (elem->max_width == 0 ?
-		details.entry->payee : truncated(details.entry->payee,
-						 elem->max_width));
+		details.entry->payee : truncate(details.entry->payee,
+						elem->max_width));
       break;
 
     case element_t::OPT_NOTE:
@@ -428,20 +660,35 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
     case element_t::NOTE:
       if (details.xact)
 	out << (elem->max_width == 0 ?
-		details.xact->note : truncated(details.xact->note,
-					       elem->max_width));
+		details.xact->note : truncate(details.xact->note,
+					      elem->max_width));
       break;
 
+    case element_t::OPT_ACCOUNT:
+      if (details.entry && details.xact) {
+	transaction_t::state_t state;
+	if (! details.entry->get_state(&state))
+	  switch (details.xact->state) {
+	  case transaction_t::CLEARED:
+	    name = "* ";
+	    break;
+	  case transaction_t::PENDING:
+	    name = "! ";
+	    break;
+	  }
+      }
+      // fall through...
+
     case element_t::ACCOUNT_NAME:
     case element_t::ACCOUNT_FULLNAME:
       if (details.account) {
-	std::string name = (elem->type == element_t::ACCOUNT_FULLNAME ?
-			    details.account->fullname() :
-			    partial_account_name(*details.account));
+	name += (elem->type == element_t::ACCOUNT_FULLNAME ?
+		 details.account->fullname() :
+		 partial_account_name(*details.account));
 
 	if (details.xact && details.xact->flags & TRANSACTION_VIRTUAL) {
 	  if (elem->max_width > 2)
-	    name = truncated(name, elem->max_width - 2);
+	    name = truncate(name, elem->max_width - 2, true);
 
 	  if (details.xact->flags & TRANSACTION_BALANCE)
 	    name = "[" + name + "]";
@@ -449,7 +696,7 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
 	    name = "(" + name + ")";
 	}
 	else if (elem->max_width > 0)
-	  name = truncated(name, elem->max_width);
+	  name = truncate(name, elem->max_width, true);
 
 	out << name;
       } else {
@@ -474,20 +721,6 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
 	}
       break;
 
-    case element_t::INTERP_FUNC:
-#ifdef USE_BOOST_PYTHON
-      try {
-	object func = python_eval(elem->chars);
-	out << call<std::string>(func.ptr(), details);
-      }
-      catch(const boost::python::error_already_set&) {
-	PyErr_Print();
-	throw format_error(std::string("While calling Python function '") +
-			   elem->chars + "'");
-      }
-#endif
-      break;
-
     default:
       assert(0);
       break;
@@ -501,9 +734,9 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
   }
 }
 
-format_transactions::format_transactions(std::ostream&      _output_stream,
+format_transactions::format_transactions(std::ostream& _output_stream,
 					 const std::string& format)
-  : output_stream(_output_stream), last_entry(NULL)
+  : output_stream(_output_stream), last_entry(NULL), last_xact(NULL)
 {
   const char * f = format.c_str();
   if (const char * p = std::strstr(f, "%/")) {
@@ -522,10 +755,16 @@ void format_transactions::operator()(transaction_t& xact)
     if (last_entry != xact.entry) {
       first_line_format.format(output_stream, details_t(xact));
       last_entry = xact.entry;
-    } else {
+    }
+    else if (last_xact && last_xact->date() != xact.date()) {
+      first_line_format.format(output_stream, details_t(xact));
+    }
+    else {
       next_lines_format.format(output_stream, details_t(xact));
     }
+
     transaction_xdata(xact).dflags |= TRANSACTION_DISPLAYED;
+    last_xact = &xact;
   }
 }
 
@@ -558,17 +797,38 @@ void format_entries::operator()(transaction_t& xact)
   last_entry = xact.entry;
 }
 
-void print_entry(std::ostream& out, const entry_t& entry)
+void print_entry(std::ostream& out, const entry_base_t& entry_base,
+		 const std::string& prefix)
 {
-  const std::string print_format
-    = "\n%D %X%C%P\n    %-34A  %12o\n%/    %-34A  %12o\n";
+  std::string print_format;
+
+  if (const entry_t * entry = dynamic_cast<const entry_t *>(&entry_base)) {
+    print_format = (prefix + "%D %X%C%P\n" +
+		    prefix + "    %-34A  %12o\n%/" +
+		    prefix + "    %-34A  %12o\n");
+  }
+  else if (const auto_entry_t * entry =
+	   dynamic_cast<const auto_entry_t *>(&entry_base)) {
+    out << "= " << entry->predicate_string << '\n';
+    print_format = prefix + "    %-34A  %12o\n";
+  }
+  else if (const period_entry_t * entry =
+	   dynamic_cast<const period_entry_t *>(&entry_base)) {
+    out << "~ " << entry->period_string << '\n';
+    print_format = prefix + "    %-34A  %12o\n";
+  }
+  else {
+    assert(0);
+  }
 
   format_entries formatter(out, print_format);
-  walk_transactions(const_cast<transactions_list&>(entry.transactions),
+  walk_transactions(const_cast<transactions_list&>(entry_base.transactions),
 		    formatter);
   formatter.flush();
 
-  clear_all_xdata();
+  clear_transaction_xdata cleaner;
+  walk_transactions(const_cast<transactions_list&>(entry_base.transactions),
+		    cleaner);
 }
 
 bool disp_subaccounts_p(const account_t&		 account,
@@ -607,7 +867,7 @@ bool disp_subaccounts_p(const account_t&		 account,
   return display;
 }
 
-bool display_account(const account_t&		      account,
+bool display_account(const account_t& account,
 		     const item_predicate<account_t>& disp_pred)
 {
   // Never display an account that has already been displayed.
@@ -657,7 +917,7 @@ format_equity::format_equity(std::ostream&      _output_stream,
 
   entry_t header_entry;
   header_entry.payee = "Opening Balances";
-  header_entry.date  = now;
+  header_entry._date = datetime_t::now;
   first_line_format.format(output_stream, details_t(header_entry));
 }
 
@@ -723,91 +983,3 @@ void format_equity::operator()(account_t& account)
 }
 
 } // namespace ledger
-
-#ifdef USE_BOOST_PYTHON
-
-#include <boost/python.hpp>
-
-using namespace boost::python;
-using namespace ledger;
-
-std::string py_format_1(format_t& format, const details_t& item)
-{
-  std::ostringstream out;
-  format.format(out, item);
-  return out.str();
-}
-
-template <typename T>
-std::string py_format(format_t& format, const T& item)
-{
-  std::ostringstream out;
-  format.format(out, details_t(item));
-  return out.str();
-}
-
-void export_format()
-{
-  typedef
-    pystream_handler_wrap<format_transactions, transaction_t, std::string>
-    format_transactions_wrap;
-
-  class_< format_transactions_wrap, bases<item_handler<transaction_t> > >
-    ("FormatTransactions",
-     init<PyObject *, std::string>()[with_custodian_and_ward<1, 2>()])
-    .def("flush", &format_transactions_wrap::flush)
-    .def("__call__", &format_transactions_wrap::operator())
-    ;
-
-  typedef
-    pystream_handler_wrap<format_entries, transaction_t, std::string>
-    format_entries_wrap;
-
-  class_< format_entries_wrap, bases<item_handler<transaction_t> > >
-    ("FormatEntries",
-     init<PyObject *, std::string>()[with_custodian_and_ward<1, 2>()])
-    .def("flush", &format_entries_wrap::flush)
-    .def("__call__", &format_entries_wrap::operator())
-    ;
-
-  typedef
-    pystream_handler_wrap<format_account, account_t, std::string, std::string>
-    format_account_wrap;
-
-  class_< format_account_wrap, bases<item_handler<transaction_t> > >
-    ("FormatAccount",
-     init<PyObject *, std::string, std::string>()
-     [with_custodian_and_ward<1, 2>()])
-    .def("flush", &format_account_wrap::flush)
-    .def("__call__", &format_account_wrap::operator())
-    ;
-
-  typedef
-    pystream_handler_wrap<format_equity, account_t, std::string, std::string>
-    format_equity_wrap;
-
-  class_< format_equity_wrap, bases<item_handler<transaction_t> > >
-    ("FormatEquity",
-     init<PyObject *, std::string, std::string>()
-     [with_custodian_and_ward<1, 2>()])
-    .def("flush", &format_equity_wrap::flush)
-    .def("__call__", &format_equity_wrap::operator())
-    ;
-
-  class_< format_t > ("Format")
-    .def(init<std::string>())
-    .def("reset", &format_t::reset)
-    .def("format", py_format_1)
-    .def("format", py_format<account_t>)
-    .def("format", py_format<entry_t>)
-    .def("format", py_format<transaction_t>)
-    ;
-
-  def("truncated", truncated);
-#if 0
-  def("partial_account_name", partial_account_name);
-#endif
-  def("display_account", display_account);
-}
-
-#endif // USE_BOOST_PYTHON
